<?php
/*
 * Copyright (c) 2012, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

abstract class WV2_RestrictArticletype_Events {
	public static function onMetaForm(array $params = array()) {
		$form   = $params['subject'];
		$select = $form->findElementByName('article_type');

		if ($select) {
			$values = $select->getValues();
			$cat    = sly_Core::getCurrentArticle()->getCategory();

			if ($cat) {
				$allowed = WV2_RestrictArticletype_Helper::getAllowedTypes($cat);

				foreach ($values as $key => $value) {
					if (!in_array($key, $allowed)) $select->removeValue($key);
				}
			}
		}

		return $form;
	}

	public static function onMetaFormAdditional(array $params = array()) {
		$form    = $params['subject'];
		$selectA = $form->findElementByName('category_copy_id_new');
		$selectB = $form->findElementByName('category_id_new');
		$changes = false;

		if ($selectA || $selectB) {
			$values  = $selectA ? $selectA->getValues() : $selectB->getValues();
			$article = sly_Core::getCurrentArticle();
			$id      = $article->getId();
			$type    = $article->getType();
			$indent  = -1;

			// If we have the 'move category' select, then we not only have to
			// check if this article's type is allowed in the target category,
			// but whether *all* used types of this category are OK.

			if ($selectB && $article->getStartpage()) {
				$sql   = WV_SQL::getInstance();
				$types = $sql->getArray('SELECT `type` FROM ~article WHERE path LIKE ? GROUP BY `type`', "%|$id|%", '~');
			}

			foreach ($values as $catID => $catName) {
				if ($catID === 0) continue;

				$i = strlen($catName) - strlen(ltrim($catName));

				if ($indent !== -1 && $i > $indent) {
					$selectA && $selectA->removeValue($catID);
					$selectB && $selectB->removeValue($catID);
					continue;
				}

				$indent  = -1;
				$allowed = WV2_RestrictArticletype_Helper::getAllowedTypes(sly_Util_Category::findById($catID));

				if (!in_array($type, $allowed)) {
					$selectA && $selectA->removeValue($catID);
					$selectB && $selectB->removeValue($catID);

					$indent  = $i;
					$changes = true;
				}
				elseif (isset($types) && count(array_diff($types, $allowed)) > 0) {
					$selectB->removeValue($catID);
					$changes = true;
					// Do not set $indent or else we would kill values in $selectA that should not be killed.
				}
			}

			if ($changes) {
				$text = 'In der Liste sind diejenigen Kategorien nicht enthalten, die diesen Artikeltyp nicht enthalten dürfen.';

				$selectA && $selectA->setHelpText($text);
				$selectB && $selectB->setHelpText($text);
			}
		}

		return $form;
	}

	public static function updateCategories(array $params = array()) {
		$catID     = (int) $params['category_id'];
		$types     = $params['types'];
		$forbidden = $params['forbidden'];

		// Find all categories within this on that have one of the restricted
		// categories in their own restriction list. We're about to remove the
		// duplicates.

		$filter = new WV_Filter_And();
		$filter->add(new WV_Filter_Article_CategoryID($catID, '=', true));
		$filter->add(new WV2_Filter_Metainfo(WV2_RestrictArticletype_Helper::METAINFO));
		$filter->add(new WV_Filter_Article_CLang(sly_Core::getCurrentClang()));

		$or = new WV_Filter_Or();

		foreach ($forbidden as $type) {
			$or->add(new WV2_Filter_Value("%|$type|%", 'LIKE'));
		}

		$filter->add($or);

		// includes the current category as well!
		$categories = WV2_MetaProvider::filterCategories($filter);

		$db = sly_DB_Persistence::getInstance();
		$db->transactional(array(__CLASS__, '_updateCategories'), array($catID, $categories, $types, $forbidden));

		WV2_Metainfo::clearDataCache();
	}

	public static function _updateCategories($catID, array $categories, array $types, array $forbidden) {
		$service  = new _WV2_Service_Metadata();
		$clangs   = sly_Util_Language::findAll(true);
		$metainfo = WV2_RestrictArticletype_Helper::METAINFO;

		foreach ($categories as $category) {
			if ($category->getId() === $catID) continue;

			$list = $category->getMeta($metainfo, array());
			$list = array_diff($list, $forbidden);
			$list = empty($list) ? '' : '|'.implode('|', $list).'|';

			foreach ($clangs as $clang) {
				$service->update($category->getId(), $clang, WV2_Metainfo::TYPE_CATEGORY, $metainfo, $list);
			}
		}

		// set the value for the current language on this category for *all* languages
		$types = empty($types) ? '' : '|'.implode('|', $types).'|';

		foreach ($clangs as $clang) {
			$service->update($catID, $clang, WV2_Metainfo::TYPE_CATEGORY, $metainfo, $types);
		}
	}
}
